require "net/ldap"
require "net/ldap/dn"

module HTH
  module Ldap
    class Reader

      def initialize(options)
        @options = options
      end

      def ldap
        @ldap ||= begin
          settings = {
            host: @options[:host],
            port: @options[:port],
            auth: {
              method: :simple,
              username: @options[:auth_username],
              password: @options[:auth_password]
            }
          }

          if use_encryption?
            settings[:encryption] = {
              method: @options[:auth_method].to_sym
            }
          end

          ldap = Net::LDAP.new(settings)
          unless ldap.bind
            raise "Authentication to LDAP server failed: " +
                  "#{ldap.get_operation_result.code}: #{ldap.get_operation_result.message}"
          end
          ldap
        end
      end

      def get_ldap_groups
        group_base = @options[:group_base]

        query_result = []

        ldap.open do |connection|
          search_options = {
            return_result: true,
            base: group_base,
            filter: groups_filter,
            attributes: ldap_group_attribute_names,
            operation: "Finding groups"
          }

          query_result = perform_search(connection, search_options)
        end

        [].tap do |result|
          query_result.each do |object|
            object_hash = {}

            @options[:ldap_group_attributes].each do |key, value|
              object_hash[key] = get_single_value(object, value)
            end

            result << object_hash
          end

        end
      end

      def use_encryption?
        %w(simple_tls start_tls).include?(@options[:auth_method].to_s)
      end

      def find_users
        users = []

        search_options = {
          return_result: true,
          base: @options[:base],
          filter: users_filter,
          attributes: ldap_attribute_names,
          operation: "Finding users"
        }

        ldap.open do |connection|
          perform_search(connection, search_options).each do |entry|
            user = decode_ldap_entry(entry)
            unless ignored_user?(user[:short_name])
              user[:ldap_groups] = find_groups_for_user(entry, connection)
              users << user
            end
          end
        end

        users
      end

      def find_users_by_ids(short_names)
        users = []
        user_entries = []

        ldap.open do |connection|
          short_names.each_slice(@options[:ldap_batch_size]) do |slice|
            user_entries += perform_search(connection, {
              return_result: true,
              base: @options[:base],
              filter: short_names_filter(slice),
              attributes: ldap_attribute_names,
              operation: "Finding users by ID"
            })
          end

          user_entries.each do |entry|
            user = decode_ldap_entry(entry)
            user[:ldap_groups] = find_groups_for_user(entry, connection)
            users << user
          end
        end

        users
      end

      def find_groups_for_user(user_entry, connection)
        groups = []
        if @options[:request_user_groups]
          search_options = {
            return_result: true,
            base: @options[:group_base],
            filter: group_member_filter(user_entry),
            attributes: %w(dn),
            operation: "Finding groups for user"
          }
          groups = perform_search(connection, search_options).map(&:dn)
        else
          groups = get_multi_value(user_entry, attribute_mappings[:ldap_groups])
        end

        parse_groups(groups)
      end

      def parse_groups(groups)
        (groups + groups.map { |group| parse_cn(group) }).uniq
      end

      def parse_cn(dn)
        begin
          ::Net::LDAP::DN.new(dn).to_a[1]
        rescue RuntimeError
          puts "Could not to parse common name from DN, using: #{dn}"
          dn
        end
      end

      def perform_search(connection, options)
        operation = options.delete(:operation)
        result  = connection.search(options)
        code    = connection.get_operation_result.code
        message = connection.get_operation_result.message
        raise "#{operation} LDAP operation failed: #{code}: #{message}" if code != 0
        result
      end

      def group_member_filter(entry)
        case attribute_mappings[:ldap_groups].downcase
        when 'member'
          Net::LDAP::Filter.eq('member', entry.dn)
        when 'member:1.2.840.113556.1.4.1941:'
          Net::LDAP::Filter.ex('member:1.2.840.113556.1.4.1941', entry.dn)
        when 'uniquemember'
          Net::LDAP::Filter.eq('uniqueMember', entry.dn)
        when 'memberuid'
          Net::LDAP::Filter.eq('memberUid', get_single_value(entry, attribute_mappings[:short_name]))
        else
          raise "Unknown group member attribute: #{attribute_mappings[:ldap_groups]}"
        end
      end

      def users_filter
        ldap_attributes = @options[:ldap_required_attributes]

        filters = Net::LDAP::Filter.present?(ldap_attributes[:short_name]) &
                  Net::LDAP::Filter.present?(ldap_attributes[:first_name]) &
                  Net::LDAP::Filter.present?(ldap_attributes[:last_name]) &
                  Net::LDAP::Filter.present?(ldap_attributes[:email])

        add_query_filters(filters)
      end

      def groups_filter
        ldap_attributes = @options[:ldap_group_attributes]

        filters = Net::LDAP::Filter.present?(ldap_attributes[:id]) &
                  Net::LDAP::Filter.present?(ldap_attributes[:name])

        add_group_query_filters(filters)
      end

      def short_names_filter(short_names)
        filters = short_names.map do |short_name|
          Net::LDAP::Filter.eq(@options[:ldap_required_attributes][:short_name], short_name)
        end.reduce(&:|)

        add_query_filters(filters)
      end

      def add_query_filters(filters)
        if @options[:query_filters]
          filters = Net::LDAP::Filter.construct(@options[:query_filters]) & filters
        end
        filters
      end

      def add_group_query_filters(filters)
        if @options[:group_query_filters]
          filters = Net::LDAP::Filter.construct(@options[:group_query_filters]) & filters
        end

        filters
      end

      def ldap_attribute_names
        attrs = []
        attrs << @options[:disabled_status][:attribute] if active_status_configured?
        attrs += @options[:ldap_required_attributes].values
        attrs += ldap_optional_attributes.values
        attrs.map(&:to_s)
      end

      def ldap_group_attribute_names
        attrs = []
        attrs += @options[:ldap_group_attributes].values
        attrs.map(&:to_s)
      end

      def ldap_optional_attributes
        @options[:ldap_optional_attributes] || {}
      end

      def decode_ldap_entry(entry)
        result = {}
        synchronized_fields = []

        attribute_mappings.each do |hth_key, ldap_key|
          next if hth_key == :ldap_groups

          if value = get_single_value(entry, ldap_key)
            result[hth_key] = value
            synchronized_fields << hth_key.to_s
          end
        end

        if result[:password]
          result[:pre_hash] = password_pre_hash(result[:password])
          synchronized_fields << "pre_hash"
        end

        result[:id]                  = result[:short_name]
        result[:active]              = active_status(entry)
        result[:synchronized_fields] = synchronized_fields

        return result
      end

      def ignored_user?(short_name)
        (@options[:ignore_users] || []).include?(short_name)
      end

      def attribute_mappings
        @options[:ldap_required_attributes].merge(ldap_optional_attributes)
      end

      def get_single_value(entry, key)
        value = entry[key]
        return nil if value.empty?
        value.first.to_s.force_encoding('utf-8')
      end

      def get_multi_value(entry, key)
        value = entry[key]
        value.map {|s| s.to_s.force_encoding('utf-8') }
      end

      # Passwords might be plaintext or pre hashed
      def password_pre_hash(ldap_password)
        if ldap_password =~ /\A\{MD5\}/
          :ldap_md5
        elsif ldap_password =~ /\A\{SHA\}/
          :ldap_sha1
        else
          nil
        end
      end

      # Figure out if the user is disabled or enabled
      def active_status(entry)
        status = true
        return status unless active_status_configured?
        active_values = entry[@options[:disabled_status][:attribute]]

        if @options[:disabled_status][:type] == 'bitmask'
          # field is a bitmask, user is disabled if a specific bit is set
          bit = @options[:disabled_status][:value]
          status = (active_values && active_values.first.to_i & bit == bit) ? false : true
        else
          # field is a normal field, user is disabled if it contains the value
          if active_values && active_values.include?(@options[:disabled_status][:value])
            status = false
          end
        end

        return status
      end

      def active_status_configured?
        @options[:disabled_status].is_a?(Hash) &&
          !@options[:disabled_status][:attribute].nil? &&
          !@options[:disabled_status][:type].nil? &&
          !@options[:disabled_status][:value].nil?
      end

    end
  end
end
