require 'net/http'
require 'multi_json'

module HTH
  class RestUserManager

    def initialize(options)
      @account_key = options[:account_key]
      @company_key = options[:company_key]
      @source      = options[:source]
      @url         = options[:api_url]
      @server_cert = options[:api_server_certificate]
      @batch_size  = options[:hth_batch_size]
      @force_sync  = options[:force_sync]
    end

    def user_count
      handle_request(uri("users/count"), :get).tap do |response|
        if response[:status] != 200
          raise "Helix TeamHub connection failure #{response}"
        end
      end
    end

    # This might send multiple requests if the number of users in the company
    # exceeds the the max limit in the API.
    def find_all(options = {})
      total_users = user_count[:content][:count]
      limit       = 500
      offset      = 0

      statuses = [200]
      contents = []

      # calculate how many requests are needed to get all the users
      request_count = request_count(total_users, limit)

      # send requests and combine data
      (1..request_count).each do |i|
        url = "users?limit=#{limit}&offset=#{offset}"
        url = "#{url}&active=#{options[:only_active] ? 'true' : 'true,false'}"
        url = "#{url}&source=#{@source}"  unless @force_sync

        data   = handle_request(uri(url), :get)
        offset = offset + limit

        statuses << data[:status]
        contents += data[:content][:results]
      end

      # return combined content with error status if any failed or success status if all ok
      return {
        status:  statuses.uniq.sort.last,
        content: contents
      }
    end

    def find_all_groups(options = {})
      limit       = 200
      offset      = 0

      statuses = [200]
      contents = []

      # send requests and combine data
      while true
        url = "groups?limit=#{limit}&offset=#{offset}"

        data   = handle_request(uri(url), :get)
        offset = offset + limit

        statuses << data[:status]
        contents += data[:content][:results]

        break if data[:content][:results].empty? or data[:content][:results].size < limit
      end

      # return combined content with error status if any failed or success status if all ok
      return {
        status:  statuses.uniq.sort.last,
        content: contents
      }
    end

    def create_user(params)
      body = encode_data({"user" => params})
      handle_request(uri('users'), :post, body)
    end

    def find_user(short_name)
      handle_request(uri("users/#{short_name}"), :get)
    end

    def update_user(params)
      body = encode_data({"user" => params})
      handle_request(uri("users/#{params[:short_name]}"), :put, body)
    end

    def delete_user(short_name)
      handle_request(uri("users/#{short_name}"), :delete)
    end

    def batch_create(users)
      split_body(users).map do |u|
        body = encode_data({batch: {batch_action: :create, data: u}})
        handle_request(uri("batch/users"), :post, body)
      end
    end

    def batch_update(users)
      split_body(users).map do |u|
        body = encode_data({batch: {batch_action: :update, data: u}})
        handle_request(uri("batch/users"), :post, body)
      end
    end

    def batch_delete(user_names)
      split_body(user_names).map do |u|
        body = encode_data({batch: {batch_action: :delete, data: u}})
        handle_request(uri("batch/users"), :post, body)
      end
    end

    def batch_group_create(groups)
      groups.map do |u|
        body = encode_group_data(u)
        handle_request(uri("groups"), :post, body)
      end
    end

    protected

      def split_body(body)
        body.each_slice(@batch_size)
      end

      def uri(path)
        URI("#{@url}/api/#{path}")
      end

      def authorization_string
        %Q{company_key="#{@company_key}",account_key="#{@account_key}"}
      end

      def handle_request(uri, method, body=nil)
        decode_data(send_request(uri, method, body))
      end

      def send_request(uri, method=:get, body=nil)
        http = Net::HTTP.new(uri.host, uri.port)

        if uri.scheme == 'https'
          http.use_ssl = true
          if @server_cert
            http.ca_file = @server_cert
            http.verify_mode = OpenSSL::SSL::VERIFY_PEER
          else
            http.verify_mode = OpenSSL::SSL::VERIFY_NONE
          end
        end

        request = case method
          when :get    then Net::HTTP::Get.new(uri.request_uri)
          when :post   then Net::HTTP::Post.new(uri.request_uri)
          when :put    then Net::HTTP::Put.new(uri.request_uri)
          when :delete then Net::HTTP::Delete.new(uri.request_uri)
        end

        request["Content-Type"]  = "application/json"
        request["Accept"]        = "application/json"
        request["Authorization"] = authorization_string
        request.body = body unless body.nil?

        response = http.request(request)

        return response
      end

      def encode_group_data(attrs)
        encode_data(attrs.merge(
          visibility: "private"
        ))
      end

      def encode_data(data)
        MultiJson.encode(data)
      end

      def decode_data(response)
        return { status: response.code.to_i, content: MultiJson.decode(response.body, symbolize_keys: true) }
      end

      def request_count(total, limit)
        request_count = total / limit
        request_count += 1  if total % limit > 0
        request_count
      end
  end
end
