require 'securerandom'

module HTH
  class Utils

    def self.parse_config(default_options)

      unless !default_options[:config_file].nil? && File.file?(default_options[:config_file])
        abort "Configuration file is required. See usage with --help"
      end

      begin
        options = YAML.load_file(default_options[:config_file])
      rescue Exception => e
        abort "Failed to parse configuration file: options #{e}."
      end

      unless options.is_a? Hash
        abort "Configurations must be a hash."
      end

      if options[:extend_config]
        begin
          base_options = YAML.load_file(File.join(SYNCHER_ROOT, 'config', options[:extend_config]))
          options = base_options.merge(options)
        rescue Exception => e
          abort "Failed to parse base configuration file: options #{e}."
        end
      end

      options = default_options.merge(options)
      options = self.set_defaults(options)
      self.ensure_source_is_set(options)
      self.ensure_valid_query_filters(options)
      self.ensure_valid_group_query_filters(options)
      self.ensure_valid_group_source(options)

      options[:reader] = 'ldap'  unless options[:reader]

      return options
    end

    def self.print_with_time(start_time, text="Running time")
      puts "\n#{text}: #{Time.now - start_time}"
    end

    def self.show(array)
      array.each do |item|
        puts "\n"
        PP.pp(item, $>, 200)
      end
    end

    protected

      # Provide backwards compatibility for new configuration options
      def self.set_defaults(options)
        unless options[:ldap_encrypt].nil?
          puts 'WARNING: Configuration option :ldap_encrypt has been deprecated, please revise your configuration.'
          options[:auth_method] = options[:ldap_encrypt] ? :simple_tls : :plain
        end

        if options[:strategy].nil?
          options[:strategy] = 'sync_all'
          puts "WARNING: Configuration option :strategy is missing, please revise your configuration. Using default value: #{options[:strategy]}"
        end

        if options[:ldap_batch_size].nil?
          options[:ldap_batch_size] = 100
          puts "WARNING: Configuration option :ldap_batch_size is missing, please revise your configuration. Using default value: #{options[:ldap_batch_size]}"
        end

        if options[:hth_batch_size].nil?
          options[:hth_batch_size] = 50
          puts "WARNING: Configuration option :hth_batch_size is missing, please revise your configuration. Using default value: #{options[:hth_batch_size]}"
        end

        options
      end

      def self.ensure_source_is_set(options)
        if options[:source].nil? || options[:source] !~ /[^[:space:]]/
          abort "You must specify unique source identifier in the config file, example:\n:source: ldap-server1"
        end
      end

      def self.ensure_valid_query_filters(options)
        if options[:query_filters].nil? || options[:query_filters].strip.empty?
          options[:query_filters] = nil
          return
        end

        begin
          Net::LDAP::Filter.construct(options[:query_filters])
        rescue Net::LDAP::FilterSyntaxInvalidError
          abort "Please check the syntax for query filters: #{options[:query_filters]}"
        end
      end

      def self.ensure_valid_group_query_filters(options)
        if options[:group_query_filters].nil? || options[:group_query_filters].strip.empty?
          options[:group_query_filters] = nil
          return
        end

        begin
          Net::LDAP::Filter.construct(options[:group_query_filters])
        rescue Net::LDAP::FilterSyntaxInvalidError
          abort "Please check the syntax for group query filters: #{options[:group_query_filters]}"
        end
      end

      def self.ensure_valid_group_source(options)
        if options[:group_sync] && !%w(cn dn).include?(options[:ldap_group_attributes][:source])
          raise ArgumentError, "group source attribute must be either cn or dn."
        end
      end

  end
end
