require 'spec_helper'

describe "Syncing all LDAP users to Helix TeamHub" do

  def initialize_dependencies(reader='ldap')
    @rest_manager = stub_everything('rest_manager')
    @source_reader = stub('ldap_reader')
    @source_reader.stubs(find_users: @source_users)
    @rest_manager.stubs(find_all: {status: 200, content: @hth_users})
    @syncher = HTH::UserSyncher.new({apply: true, reader: reader, source: :test_source}, ldap_reader: @source_reader, rest_user_manager: @rest_manager)
  end

  before(:each) do
    @user1 = build_user
    @user2 = build_user
    @user3 = build_user
  end

  context "finding users from Helix TeamHub" do
    before(:each) do
      @hth_users = @source_users = []
      initialize_dependencies()
    end

    it "requests only active users" do
      @rest_manager.expects(:find_all).with(only_active: true).
        returns({status: 200, content: @hth_users})
      @syncher.sync_all
    end
  end

  context "with only new users" do
    before(:each) do
      @hth_users = []
      @source_users = [@user1, @user2, deactivate_source_user(@user3)]

      initialize_dependencies()
    end

    it "does not update any users" do
      @rest_manager.expects(:batch_update).never
      @syncher.sync_all
    end

    it "creates only active users" do
      @rest_manager.expects(:batch_create).with([@user1, @user2]).once
      @syncher.sync_all
    end

    it "does not delete users" do
      @rest_manager.expects(:batch_delete).never
      @syncher.sync_all
    end
  end

  context "with no new users" do
    before(:each) do
      @hth_users = [@user1]
      @source_users = [@user1, deactivate_source_user(@user2)]
      initialize_dependencies
    end

    it "updates only active users" do
      @rest_manager.expects(:batch_update).with([@user1]).once
      @syncher.sync_all
    end

    it "does not create users" do
      @rest_manager.expects(:batch_create).never
      @syncher.sync_all
    end

    it "does not delete users" do
      @rest_manager.expects(:batch_delete).never
      @syncher.sync_all
    end
  end

  context "with some new users" do
    before(:each) do
      @hth_users = [@user1]
      @source_users = [@user1, @user2]

      initialize_dependencies
    end

    it "updates existing users" do
      @rest_manager.expects(:batch_update).with([@user1]).once
      @syncher.sync_all
    end

    it "creates new users" do
      @rest_manager.expects(:batch_create).with([@user2]).once
      @syncher.sync_all
    end

    it "does not delete users" do
      @rest_manager.expects(:batch_delete).never
      @syncher.sync_all
    end
  end

  context "when deactivated hth users is no longer in source" do
    before do
      @hth_users  = [deactivate_hth_user(@user2)]
      @source_users = []

      initialize_dependencies
    end

    it "does not try to deactivate already deactivated hth users" do
      @rest_manager.expects(:batch_delete).never
      @syncher.sync_all
    end
  end

  context "with some obsolete users" do
    before(:each) do
      @hth_users = [@user1, @user2]
      @source_users = [@user1, deactivate_source_user(@user2), deactivate_source_user(@user3)]

      initialize_dependencies
    end

    it "updates active users" do
      @rest_manager.expects(:batch_update).with([@user1]).once
      @syncher.sync_all
    end

    it "does not create users" do
      @rest_manager.expects(:batch_create).never
      @syncher.sync_all
    end

    it "deletes obsolete users" do
      @rest_manager.expects(:batch_delete).with([@user2[:short_name]]).once
      @syncher.sync_all
    end
  end

  context "with deleted source users" do
    before do
      @hth_users = [@user1, @user2]
      @source_users = [@user1]

      initialize_dependencies
    end

    it "creates no new users" do
      @rest_manager.expects(:batch_create).never
      @syncher.sync_all
    end

    it "updates existing hth user" do
      @rest_manager.expects(:batch_update).with([@user1]).once
      @syncher.sync_all
    end

    it "removes obsolete hth user" do
      @rest_manager.expects(:batch_delete).with([@user2[:short_name]]).once
      @syncher.sync_all
    end
  end

  context "with some errors" do
    before(:each) do
      @hth_users = [@user1, @user3]
      @source_users = [@user1, @user2]

      batch_result = [{status: 400, content: [
          {status: 400, content: {password: 'foobar'}},
          {status: 400, content: {password: 'quux'}}
      ]}]

      initialize_dependencies

      @rest_manager.stubs(batch_update: batch_result, batch_create: batch_result)
    end

    it "saves the errors" do
      @syncher.sync_all

      errors = @syncher.errors

      errors[:update].should_not be_empty
      errors[:create].should_not be_empty
    end
  end
end
